
SCANDIRHELPER_REPR = '''
------------------------------------
Filename:    {filename}
iNode:       {inode}
Path:        {path}
Permissions: {perms}
Owner:       {owner}
Group:       {group}
Size:        {size}
Created:     {created}
Modified:    {modified}'''


FILELIST_FILE_DETAILS = '''
{filename}

   {perms}
   {size}
   created:  {created}
   modified: {modified}
'''


HELP_TEXT = '''
         Welcome to FILER

 [Up/Down Arrows]  Navigate list
      ["] and [?]  Jump up/down list (20 lines)
     [Left Arrow]  Go to parent directory
    [Right Arrow]  Go into directory
          [Enter]  Actions menu/Select action
     [B] or [Esc]  Go back
          [Space]  Select/deselect file
              [A]  Select/delect all
              [.]  Show/hide hidden files
              [D]  Show/hide directories
              [F]  Show/hide non-directory files
              [H]  Show help (this window)
              [Q]  Quit'''